package me.armar.plugins.autorank;

import net.milkbowl.vault.Vault;
import net.milkbowl.vault.permission.Permission;

import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.util.LinkedHashMap;
import java.util.logging.Logger;

public class Autorank extends JavaPlugin {

	private Logger log = Logger.getLogger("Minecraft");
	private RankChecker rankChecker;
	private String configPath;
	private String dataPath;
	private Config config;
	private Config data;
	private LinkedHashMap<String, Object> configDefaults = new LinkedHashMap<String, Object>();
	private VaultHandler vault;
	public static Permission permission = null;
	private AutorankUpdateData update;
	private AutorankSaveData save;
	private Leaderboard leaderboard;
	private AutorankUpdateLeaderboard leaderboardUpdate;
	private boolean debug;

	public void onEnable() {
		// check if vault is present
		Plugin x = this.getServer().getPluginManager().getPlugin("Vault");
		if (x != null & x instanceof Vault) {
			vault = new VaultHandler(this);
		} else {
			logMessage("[%s] Vault was _NOT_ found! Disabling autorank.");
			getPluginLoader().disablePlugin(this);
		}

		// make leaderboard
		leaderboard = new Leaderboard();

		// set up general config
		this.configPath = this.getDataFolder().getAbsolutePath()
				+ File.separator + "config.yml";
		this.configDefaults.put("Automatic rank changes", false);
		this.configDefaults.put("Debug mode", false);
		this.configDefaults.put("Update interval(minutes)", 5);
		this.configDefaults.put("Leaderboard update interval(minutes)", 30);
		this.configDefaults.put("Log updates", false);
		this.configDefaults.put("Save interval(minutes)", 60);
		this.configDefaults.put("Starting rank", "Newcomer");
		this.configDefaults.put("Rank1.name", "Member");
		this.configDefaults.put("Rank1.required minutes played", 1800);
		this.configDefaults.put("Rank1.message",
				"Congratulations, you are now a Member.");
		this.configDefaults.put("Rank2.name", "Veteran");
		this.configDefaults.put("Rank2.required minutes played", 12000);
		this.configDefaults.put("Rank2.message",
				"Congratulations, you are now a Veteran.");
		config = new Config(this, configPath, configDefaults, "config");

		// set up player data
		this.dataPath = this.getDataFolder().getAbsolutePath() + File.separator
				+ "data.yml";
		data = new Config(this, dataPath, null, "data");

		rankChecker = new RankChecker(this, config, data);
		// schedule AutorankUpdate to be run
		update = new AutorankUpdateData(this);
		int updateInterval = 60 * 20 * (Integer) config
				.get("Update interval(minutes)");
		getServer().getScheduler().scheduleAsyncRepeatingTask(this, update,
				updateInterval, updateInterval);

		// schedule AutorankSave to be run
		save = new AutorankSaveData(this);
		int saveInterval = 60 * 20 * (Integer) config
				.get("Save interval(minutes)");
		getServer().getScheduler().scheduleAsyncRepeatingTask(this, save,
				saveInterval, saveInterval);

		// schedule AutorankUpdateLeaderboard to be run
		if (config.get("Leaderboard update interval(minutes)") == null) {
			config.set("Leaderboard update interval(minutes)", 30);
			logMessage("Updated config to include Leaderboard update interval");
			getConf().save();
		}
		leaderboardUpdate = new AutorankUpdateLeaderboard(this);
		int leaderboardUpdateInterval = 60 * 20 * (Integer) config
				.get("Leaderboard update interval(minutes)");
		getServer().getScheduler().scheduleAsyncRepeatingTask(this,
				leaderboardUpdate, 0, leaderboardUpdateInterval);

		// get debug value
		if (config.get("Debug mode") != null) {
			debug = (Boolean) config.get("Debug mode");
		}

		logMessage("Enabled.");

	}

	public void onDisable() {
		getData().save();
		logMessage("Data saved");
		logMessage("Disabled.");
		// stop scheduled tasks from running
		getServer().getScheduler().cancelTasks(this);
	}

	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		String noPerm = "You do not have permission to use this command.";
		boolean overridePerms = sender.hasPermission("autorank.*");
		if (sender.hasPermission("autorank.commands")) {
			overridePerms = true;
		}
		// onCommand returns true if the command was handled by the plugin
		if (args.length == 1) {
			if (args[0].equalsIgnoreCase("help")) {
				sender.sendMessage("Actions: check, check [name], change [name], leaderboard, set [name] [value]");
			} else if (args[0].equalsIgnoreCase("leaderboard")) {
				// check permissions
				if (!sender.hasPermission("autorank.leaderboard")
						&& !overridePerms) {
					sender.sendMessage(noPerm);
					return true;
				}
				leaderboard.display(sender);
				return true;
			}
			if (args[0].equalsIgnoreCase("check")) {
				if (!sender.hasPermission("autorank.check") && !overridePerms) {
					sender.sendMessage(noPerm);
					return true;
				}
				if (sender instanceof ConsoleCommandSender) {
					sender.sendMessage("Cannot check for console.");
					return true;
				}
				sender.sendMessage(rankChecker.toString(sender.getName()));
				return true;
			}
		}
		if (args.length > 1) {
			// set variables
			String playerName = args[1].toLowerCase();
			Player player = getServer().getPlayer(playerName);
			// handle CHECK OTHERS command
			if (args[0].equalsIgnoreCase("check")) {
				// check permissions
				if (!sender.hasPermission("autorank.checkothers")
						&& !overridePerms) {
					sender.sendMessage(noPerm);
					return true;
				}
				// check if player is online
				if (player == null) {
					sender.sendMessage("Player is offline.");
					return true;
				}
				sender.sendMessage(rankChecker.toString(args[1]));
				return true;
				// handle CHANGE command
			}
			if (args[0].equalsIgnoreCase("change")) {
				// check permissions
				if (!sender.hasPermission("autorank.change") && !overridePerms) {
					sender.sendMessage(noPerm);
					return true;
				}
				// check if player is online
				if (player == null) {
					sender.sendMessage("Player is offline.");
					return true;
				}
				// check if player is excluded
				if (player.hasPermission("autorank.exclude")) {
					sender.sendMessage("This player is excluded from rank changes by autorank.");
					return true;
				}
				String[] playerData = rankChecker.getPlayerData(args[1]);
				// check if player is unknown
				if (playerData[0].equals("Unknown")) {
					sender.sendMessage("Unknown player.");
					return true;
				}
				// check for downranks
				if (playerData[0].equals("No upgrade")) {
					sender.sendMessage("Cannot downrank.");
					return true;
				}
				// change rank
				vault.setRank(player.getName(), playerData[1], playerData[2]);
				sender.sendMessage("Rank changed to " + playerData[2] + ".");
				return true;
				// Handle SET command
			}
			if (args[0].equalsIgnoreCase("set")) {
				// check permissions
				if (!sender.hasPermission("autorank.set") && !overridePerms) {
					sender.sendMessage(noPerm);
					return true;
				}
				try {
					int value = Integer.parseInt(args[2]);
					if (!data.exists(playerName)) {
						data.set(playerName, 0);
					}
					data.set(playerName, (Integer) value);
					sender.sendMessage(playerName
							+ "'s playtime has been set to " + value);
					return true;
				} catch (Exception e) {
					sender.sendMessage("Cannot set to that.");
					return true;
				}

			}
		}
		return false;
	}

	public void logMessage(String msg) {
		PluginDescriptionFile pdFile = this.getDescription();
		this.log.info(pdFile.getName() + " " + pdFile.getVersion() + " : "
				+ msg);
	}

	public void debugMessage(String msg) {
		if (debug) {
			this.log.info(msg);
		}
	}

	public Config getConf() {// name is weird because getConfig is used by
								// JavaPlugin
		return config;
	}

	public Config getData() {
		return data;
	}

	public VaultHandler getVault() {
		return vault;
	}

	public RankChecker getRankChecker() {
		return rankChecker;
	}

	public Leaderboard getLeaderboard() {
		return leaderboard;
	}

	public void clearLeaderboard() {
		leaderboard = new Leaderboard();
	}
}
