package me.armar.plugins.autorank;

public class RankChecker {

	private Config config;
	private Config data;
	private Autorank plugin;
	private VaultHandler vault;

	public RankChecker(Autorank plugin, Config config, Config data) {
		this.config = config;
		this.plugin = plugin;
		this.data = data;
		vault = plugin.getVault();
	}

	public String[] getPlayerData(String playerName) {
		// [0] = status (Unknown/Upgrade/No upgrade)
		// [1] = current rank
		// [2] = new rank
		playerName = playerName.toLowerCase();
		String rank = (String) config.get("Starting rank");
		String currentRank = vault.getRank(plugin.getServer().getPlayer(
				playerName));
		String startingRank = (String) config.get("Starting rank");
		String[] res = { "", currentRank, "" };
		if (data.exists(playerName)) {
			int rankNumber = 1;
			boolean upgrade = false;
			if ((startingRank == null && currentRank == null)
					|| (startingRank != null && (startingRank).equalsIgnoreCase(currentRank))) {
				upgrade = true;
			}
			boolean done = false;
			while ((Integer) config.get("Rank" + rankNumber
					+ ".required minutes played") != null && !done) {
				if ((Integer) data.get(playerName) >= (Integer) config
						.get("Rank" + rankNumber + ".required minutes played")) {
					rank = (String) config.get("Rank" + rankNumber + ".name");
				}else{
					done = true;
				}
				rankNumber++;
				if (!upgrade && (rank != null && rank.equals(currentRank))) {
					upgrade = true;
				}
			}
			if ((rank == null && currentRank == null)
					|| (rank != null && rank.equals(currentRank))) {
				upgrade = false;
			}
			res[2] = rank;
			if (upgrade) {
				res[0] = "Upgrade";
				plugin.debugMessage(res[0] + ", " + res[1] + ", " + res[2]);
				return res;
			} else {
				res[0] = "No upgrade";
				plugin.debugMessage(res[0] + ", " + res[1] + ", " + res[2]);
				return res;
			}
		} else {
			res[0] = "Unknown";
			plugin.debugMessage(res[0] + ", " + res[1] + ", " + res[2]);
			return res;
		}
	}

	public String toString(String playerName) {
		String res = "";
		playerName = playerName.toLowerCase();
		if (data.exists(playerName)) {
			String playerData[] = getPlayerData(playerName);
			int time = (Integer) data.get(playerName);
			int seconds = time;
			int days = time / 1440;
			time -= days * 1440;
			int hours = time / 60;
			res += playerData[1] + " " + playerName + " has been online for " + seconds
					+ " minutes or " + days + " days and " + hours + " hours. ";

			if (!(Boolean) config.get("Automatic rank changes")) {
				if (playerData[0].equals("No upgrade")) {
					res += "This player's rank of " + playerData[1]
							+ " does not need to be changed. ";
				} else {
					res += "The recommended rank for this player is "
							+ playerData[2] + ".";
				}
			} else {

				int rankNumber = getRankNumber(playerData[1]);
				if (rankNumber != -1) {
					rankNumber++;
					if (config.get("Rank" + rankNumber
							+ ".required minutes played") != null) {
						int minsLeft = (Integer) config.get("Rank" + rankNumber
								+ ".required minutes played")
								- seconds;
						res += "This player will be ranked up to "
								+ config.get("Rank" + rankNumber + ".name") + " after " + minsLeft
								+ " more minutes.";
					}
				} else {
					res += "This player's rank of " + playerData[1]
							+ " does not need to be changed. ";

				}
			}

		} else {
			return "Unknown player";
		}
		return res;
	}

	public int getRankNumber(String rankName) {
		if (rankName == null) {
			return 0;
		}
		int rankNumber = 1;
		boolean found = false;
		while (!found && config.get("Rank" + rankNumber + ".name") != null) {
			if (rankName.equals(config.get("Rank" + rankNumber + ".name"))) {
				found = true;
			} else {
				rankNumber++;
			}
		}
		if (found) {
			plugin.debugMessage("ranknumber: " + rankNumber);
			return rankNumber;
		} else {
			if ((rankName == null && config.get("Starting rank") == null) || (rankName != null && (rankName)
					.equalsIgnoreCase((String) config.get("Starting rank")))) {
				plugin.debugMessage("Starting rank");
				return 0;
			}
			plugin.debugMessage("Wrong rank");
			return -1;
		}
	}

}